@@ -109,6 +109,10 @@ void
 janitor_threadclass_setdetached(janitor_threadclass_t threadclass)
 {
     threadclass->detached = 1;
+    if (!threadclass->hasattr) {
+        pthread_attr_init(&threadclass->attr);
+    }
+    pthread_attr_setdetachstate(&threadclass->attr, PTHREAD_CREATE_DETACHED);
 }
 
 void
@@ -127,7 +131,9 @@ void
 janitor_threadclass_setminstacksize(janitor_threadclass_t threadclass, size_t minstacksize)
 {
     size_t stacksize;
-    pthread_attr_init(&threadclass->attr);
+    if (!threadclass->hasattr) {
+        pthread_attr_init(&threadclass->attr);
+    }
     pthread_attr_getstacksize(&threadclass->attr, &stacksize);
     if (stacksize < minstacksize) {
         pthread_attr_setstacksize(&threadclass->attr, minstacksize);
@@ -241,7 +247,7 @@ janitor_thread_dispose(janitor_thread_t info)
         }
         pthread_mutex_unlock(&threadlock);
     }
-    /*free(info);*/
+    free(info);
 }
 
 static void
@@ -276,6 +282,9 @@ janitor_thread_finished(janitor_thread_t info)
         }
         finishedthreadlist = info;
         pthread_mutex_unlock(&threadlock);
+    } else {
+        pthread_detach(pthread_self());
+        janitor_thread_dispose(info);
     }
 }
 
@@ -308,7 +317,7 @@ runthread(void* data)
     }
     info->runfunc(info->rundata);
     sigaltstack(&prevss, NULL);
-    /*free(ss.ss_sp);*/ //Can't free this, libxml tries to access it?
+    free(ss.ss_sp); /* libxml has/had problems when freeing this as it tries to access it */
     janitor_thread_unregister(info);
     janitor_thread_finished(info);
     return NULL;
@@ -387,7 +396,6 @@ janitor_thread_tryjoinall(janitor_threadclass_t threadclass)
         }
         pthread_mutex_unlock(&threadlock);
         if (foundthread) {
-            free(foundthread->rundata);
             janitor_thread_join(foundthread);
         }
     } while(foundthread);
@@ -657,6 +665,13 @@ char*
 janitor_backtrace_string(void)
 {
     char* string = NULL;
+#ifdef HAVE_BACKTRACE
+    Dl_info btinfo;
+    void *bt[20];
+    const char* symb;
+    int count, i, siz, len;
+#endif
+
 #ifdef HAVE_BACKTRACE_FULL
     if(frames == NULL) {
         frames = backtrace_create_state(NULL, 0, (backtrace_error_callback) errorhandler, NULL);
@@ -664,6 +679,30 @@ janitor_backtrace_string(void)
     pthread_mutex_lock(&frameslock);
     backtrace_full(frames, 1, (backtrace_full_callback) callbackstring, (backtrace_error_callback) errorhandler, &string);
     pthread_mutex_unlock(&frameslock);
+#else
+#ifdef HAVE_BACKTRACE
+    count = backtrace(bt, sizeof (bt) / sizeof (void*));
+    for (i = 1; i < count; i++) {
+        dladdr(bt[i], &btinfo);
+        if (btinfo.dli_sname != NULL) {
+            symb = btinfo.dli_sname;
+            if (!strcmp(btinfo.dli_sname, "main"))
+                break;
+        } else
+            symb = "unknown";
+        siz = 2 + strlen(symb) + 1;
+        if (string != NULL) {
+            len = strlen(string);
+            siz += len + 1;
+            string = realloc(string, siz);
+        } else {
+            len = 0;
+            siz += 1;
+            string = malloc(siz);
+        }
+        snprintf(&(string[len]), siz-len, "  %s\n", symb);
+    }
+#endif
 #endif
     return string;
 }
