@@ -54,6 +54,8 @@ typedef enum hsm_key_role {
 } hsm_key_role_t;
 extern const db_enum_t hsm_key_enum_set_role[];
 
+#define HSM_KEY_ROLE_SEP(role) ((role) == HSM_KEY_ROLE_KSK || (role) == HSM_KEY_ROLE_CSK)
+
 typedef enum hsm_key_key_type {
     HSM_KEY_KEY_TYPE_INVALID = -1,
     HSM_KEY_KEY_TYPE_RSA = 1
@@ -98,20 +100,20 @@ struct hsm_key {
  * \param[in] connection a db_connection_t pointer.
  * \return a hsm_key_t pointer or NULL on error.
  */
-hsm_key_t* hsm_key_new(const db_connection_t* connection);
+extern hsm_key_t* hsm_key_new(const db_connection_t* connection);
 
 /**
  * Create a new hsm key object that is a copy of another hsm key object.
  * \param[in] hsm_key a hsm_key_t pointer.
  * \return a hsm_key_t pointer or NULL on error.
  */
-hsm_key_t* hsm_key_new_copy(const hsm_key_t* hsm_key);
+extern hsm_key_t* hsm_key_new_copy(const hsm_key_t* hsm_key);
 
 /**
  * Delete a hsm key object, this does not delete it from the database.
  * \param[in] hsm_key a hsm_key_t pointer.
  */
-void hsm_key_free(hsm_key_t* hsm_key);
+extern void hsm_key_free(hsm_key_t* hsm_key);
 
 /**
  * Copy the content of a hsm key object.
@@ -119,7 +121,7 @@ void hsm_key_free(hsm_key_t* hsm_key);
  * \param[in] hsm_key_copy a hsm_key_t pointer.
  * \return DB_ERROR_* on failure, otherwise DB_OK.
  */
-int hsm_key_copy(hsm_key_t* hsm_key, const hsm_key_t* hsm_key_copy);
+extern int hsm_key_copy(hsm_key_t* hsm_key, const hsm_key_t* hsm_key_copy);
 
 /**
  * Set the content of a hsm key object based on a database result.
@@ -127,77 +129,77 @@ int hsm_key_copy(hsm_key_t* hsm_key, const hsm_key_t* hsm_key_copy);
  * \param[in] result a db_result_t pointer.
  * \return DB_ERROR_* on failure, otherwise DB_OK.
  */
-int hsm_key_from_result(hsm_key_t* hsm_key, const db_result_t* result);
+extern int hsm_key_from_result(hsm_key_t* hsm_key, const db_result_t* result);
 
 /**
  * Get the id of a hsm key object.
  * \param[in] hsm_key a hsm_key_t pointer.
  * \return a db_value_t pointer or NULL on error.
  */
-const db_value_t* hsm_key_id(const hsm_key_t* hsm_key);
+extern const db_value_t* hsm_key_id(const hsm_key_t* hsm_key);
 
 /**
  * Get the policy_id of a hsm key object.
  * \param[in] hsm_key a hsm_key_t pointer.
  * \return a db_value_t pointer or NULL on error.
  */
-const db_value_t* hsm_key_policy_id(const hsm_key_t* hsm_key);
+extern const db_value_t* hsm_key_policy_id(const hsm_key_t* hsm_key);
 
 /**
  * Get the locator of a hsm key object.
  * \param[in] hsm_key a hsm_key_t pointer.
  * \return a character pointer or NULL on error or if no locator has been set.
  */
-const char* hsm_key_locator(const hsm_key_t* hsm_key);
+extern const char* hsm_key_locator(const hsm_key_t* hsm_key);
 
 /**
  * Get the state of a hsm key object.
  * \param[in] hsm_key a hsm_key_t pointer.
  * \return a hsm_key_state_t which may be HSM_KEY_STATE_INVALID on error or if no state has been set.
  */
-hsm_key_state_t hsm_key_state(const hsm_key_t* hsm_key);
+extern hsm_key_state_t hsm_key_state(const hsm_key_t* hsm_key);
 
 /**
  * Get the bits of a hsm key object. Undefined behavior if `hsm_key` is NULL.
  * \param[in] hsm_key a hsm_key_t pointer.
  * \return an unsigned integer.
  */
-unsigned int hsm_key_bits(const hsm_key_t* hsm_key);
+extern unsigned int hsm_key_bits(const hsm_key_t* hsm_key);
 
 /**
  * Get the algorithm of a hsm key object. Undefined behavior if `hsm_key` is NULL.
  * \param[in] hsm_key a hsm_key_t pointer.
  * \return an unsigned integer.
  */
-unsigned int hsm_key_algorithm(const hsm_key_t* hsm_key);
+extern unsigned int hsm_key_algorithm(const hsm_key_t* hsm_key);
 
 /**
  * Get the role of a hsm key object.
  * \param[in] hsm_key a hsm_key_t pointer.
  * \return a hsm_key_role_t which may be HSM_KEY_ROLE_INVALID on error or if no role has been set.
  */
-hsm_key_role_t hsm_key_role(const hsm_key_t* hsm_key);
+extern hsm_key_role_t hsm_key_role(const hsm_key_t* hsm_key);
 
 /**
  * Get the inception of a hsm key object. Undefined behavior if `hsm_key` is NULL.
  * \param[in] hsm_key a hsm_key_t pointer.
  * \return an unsigned integer.
  */
-unsigned int hsm_key_inception(const hsm_key_t* hsm_key);
+extern unsigned int hsm_key_inception(const hsm_key_t* hsm_key);
 
 /**
  * Get the repository of a hsm key object.
  * \param[in] hsm_key a hsm_key_t pointer.
  * \return a character pointer or NULL on error or if no repository has been set.
  */
-const char* hsm_key_repository(const hsm_key_t* hsm_key);
+extern const char* hsm_key_repository(const hsm_key_t* hsm_key);
 
 /**
  * Get the backup of a hsm key object.
  * \param[in] hsm_key a hsm_key_t pointer.
  * \return a hsm_key_backup_t which may be HSM_KEY_BACKUP_INVALID on error or if no backup has been set.
  */
-hsm_key_backup_t hsm_key_backup(const hsm_key_t* hsm_key);
+extern hsm_key_backup_t hsm_key_backup(const hsm_key_t* hsm_key);
 
 /**
  * Set the policy_id of a hsm key object. If this fails the original value may have been lost.
@@ -205,7 +207,7 @@ hsm_key_backup_t hsm_key_backup(const hsm_key_t* hsm_key);
  * \param[in] policy_id a db_value_t pointer.
  * \return DB_ERROR_* on failure, otherwise DB_OK.
  */
-int hsm_key_set_policy_id(hsm_key_t* hsm_key, const db_value_t* policy_id);
+extern int hsm_key_set_policy_id(hsm_key_t* hsm_key, const db_value_t* policy_id);
 
 /**
  * Set the locator of a hsm key object.
@@ -213,7 +215,7 @@ int hsm_key_set_policy_id(hsm_key_t* hsm_key, const db_value_t* policy_id);
  * \param[in] locator_text a character pointer.
  * \return DB_ERROR_* on failure, otherwise DB_OK.
  */
-int hsm_key_set_locator(hsm_key_t* hsm_key, const char* locator_text);
+extern int hsm_key_set_locator(hsm_key_t* hsm_key, const char* locator_text);
 
 /**
  * Set the state of a hsm key object.
@@ -221,7 +223,7 @@ int hsm_key_set_locator(hsm_key_t* hsm_key, const char* locator_text);
  * \param[in] state a hsm_key_state_t.
  * \return DB_ERROR_* on failure, otherwise DB_OK.
  */
-int hsm_key_set_state(hsm_key_t* hsm_key, hsm_key_state_t state);
+extern int hsm_key_set_state(hsm_key_t* hsm_key, hsm_key_state_t state);
 
 /**
  * Set the bits of a hsm key object.
@@ -229,7 +231,7 @@ int hsm_key_set_state(hsm_key_t* hsm_key, hsm_key_state_t state);
  * \param[in] bits an unsigned integer.
  * \return DB_ERROR_* on failure, otherwise DB_OK.
  */
-int hsm_key_set_bits(hsm_key_t* hsm_key, unsigned int bits);
+extern int hsm_key_set_bits(hsm_key_t* hsm_key, unsigned int bits);
 
 /**
  * Set the algorithm of a hsm key object.
@@ -237,7 +239,7 @@ int hsm_key_set_bits(hsm_key_t* hsm_key, unsigned int bits);
  * \param[in] algorithm an unsigned integer.
  * \return DB_ERROR_* on failure, otherwise DB_OK.
  */
-int hsm_key_set_algorithm(hsm_key_t* hsm_key, unsigned int algorithm);
+extern int hsm_key_set_algorithm(hsm_key_t* hsm_key, unsigned int algorithm);
 
 /**
  * Set the role of a hsm key object.
@@ -245,7 +247,7 @@ int hsm_key_set_algorithm(hsm_key_t* hsm_key, unsigned int algorithm);
  * \param[in] role a hsm_key_role_t.
  * \return DB_ERROR_* on failure, otherwise DB_OK.
  */
-int hsm_key_set_role(hsm_key_t* hsm_key, hsm_key_role_t role);
+extern int hsm_key_set_role(hsm_key_t* hsm_key, hsm_key_role_t role);
 
 /**
  * Set the inception of a hsm key object.
@@ -253,7 +255,7 @@ int hsm_key_set_role(hsm_key_t* hsm_key, hsm_key_role_t role);
  * \param[in] inception an unsigned integer.
  * \return DB_ERROR_* on failure, otherwise DB_OK.
  */
-int hsm_key_set_inception(hsm_key_t* hsm_key, unsigned int inception);
+extern int hsm_key_set_inception(hsm_key_t* hsm_key, unsigned int inception);
 
 /**
  * Set the key_type of a hsm key object.
@@ -261,7 +263,7 @@ int hsm_key_set_inception(hsm_key_t* hsm_key, unsigned int inception);
  * \param[in] key_type a hsm_key_key_type_t.
  * \return DB_ERROR_* on failure, otherwise DB_OK.
  */
-int hsm_key_set_key_type(hsm_key_t* hsm_key, hsm_key_key_type_t key_type);
+extern int hsm_key_set_key_type(hsm_key_t* hsm_key, hsm_key_key_type_t key_type);
 
 /**
  * Set the repository of a hsm key object.
@@ -269,7 +271,7 @@ int hsm_key_set_key_type(hsm_key_t* hsm_key, hsm_key_key_type_t key_type);
  * \param[in] repository_text a character pointer.
  * \return DB_ERROR_* on failure, otherwise DB_OK.
  */
-int hsm_key_set_repository(hsm_key_t* hsm_key, const char* repository_text);
+extern int hsm_key_set_repository(hsm_key_t* hsm_key, const char* repository_text);
 
 /**
  * Set the backup of a hsm key object.
@@ -277,7 +279,7 @@ int hsm_key_set_repository(hsm_key_t* hsm_key, const char* repository_text);
  * \param[in] backup a hsm_key_backup_t.
  * \return DB_ERROR_* on failure, otherwise DB_OK.
  */
-int hsm_key_set_backup(hsm_key_t* hsm_key, hsm_key_backup_t backup);
+extern int hsm_key_set_backup(hsm_key_t* hsm_key, hsm_key_backup_t backup);
 
 /**
  * Create a clause for policy_id of a hsm key object and add it to a database clause list.
@@ -288,7 +290,7 @@ int hsm_key_set_backup(hsm_key_t* hsm_key, hsm_key_backup_t backup);
  * \param[in] policy_id a db_value_t pointer.
  * \return a db_clause_t pointer to the added clause or NULL on error.
  */
-db_clause_t* hsm_key_policy_id_clause(db_clause_list_t* clause_list, const db_value_t* policy_id);
+extern db_clause_t* hsm_key_policy_id_clause(db_clause_list_t* clause_list, const db_value_t* policy_id);
 
 /**
  * Create a clause for state of a hsm key object and add it to a database clause list.
@@ -299,7 +301,7 @@ db_clause_t* hsm_key_policy_id_clause(db_clause_list_t* clause_list, const db_va
  * \param[in] state a hsm_key_state_t.
  * \return a db_clause_t pointer to the added clause or NULL on error.
  */
-db_clause_t* hsm_key_state_clause(db_clause_list_t* clause_list, hsm_key_state_t state);
+extern db_clause_t* hsm_key_state_clause(db_clause_list_t* clause_list, hsm_key_state_t state);
 
 /**
  * Create a clause for bits of a hsm key object and add it to a database clause list.
@@ -310,7 +312,7 @@ db_clause_t* hsm_key_state_clause(db_clause_list_t* clause_list, hsm_key_state_t
  * \param[in] bits an unsigned integer.
  * \return a db_clause_t pointer to the added clause or NULL on error.
  */
-db_clause_t* hsm_key_bits_clause(db_clause_list_t* clause_list, unsigned int bits);
+extern db_clause_t* hsm_key_bits_clause(db_clause_list_t* clause_list, unsigned int bits);
 
 /**
  * Create a clause for algorithm of a hsm key object and add it to a database clause list.
@@ -321,7 +323,7 @@ db_clause_t* hsm_key_bits_clause(db_clause_list_t* clause_list, unsigned int bit
  * \param[in] algorithm an unsigned integer.
  * \return a db_clause_t pointer to the added clause or NULL on error.
  */
-db_clause_t* hsm_key_algorithm_clause(db_clause_list_t* clause_list, unsigned int algorithm);
+extern db_clause_t* hsm_key_algorithm_clause(db_clause_list_t* clause_list, unsigned int algorithm);
 
 /**
  * Create a clause for role of a hsm key object and add it to a database clause list.
@@ -332,7 +334,7 @@ db_clause_t* hsm_key_algorithm_clause(db_clause_list_t* clause_list, unsigned in
  * \param[in] role a hsm_key_role_t.
  * \return a db_clause_t pointer to the added clause or NULL on error.
  */
-db_clause_t* hsm_key_role_clause(db_clause_list_t* clause_list, hsm_key_role_t role);
+extern db_clause_t* hsm_key_role_clause(db_clause_list_t* clause_list, hsm_key_role_t role);
 
 /**
  * Create a clause for is_revoked of a hsm key object and add it to a database clause list.
@@ -343,7 +345,7 @@ db_clause_t* hsm_key_role_clause(db_clause_list_t* clause_list, hsm_key_role_t r
  * \param[in] is_revoked an unsigned integer.
  * \return a db_clause_t pointer to the added clause or NULL on error.
  */
-db_clause_t* hsm_key_is_revoked_clause(db_clause_list_t* clause_list, unsigned int is_revoked);
+extern db_clause_t* hsm_key_is_revoked_clause(db_clause_list_t* clause_list, unsigned int is_revoked);
 
 /**
  * Create a clause for key_type of a hsm key object and add it to a database clause list.
@@ -354,7 +356,7 @@ db_clause_t* hsm_key_is_revoked_clause(db_clause_list_t* clause_list, unsigned i
  * \param[in] key_type a hsm_key_key_type_t.
  * \return a db_clause_t pointer to the added clause or NULL on error.
  */
-db_clause_t* hsm_key_key_type_clause(db_clause_list_t* clause_list, hsm_key_key_type_t key_type);
+extern db_clause_t* hsm_key_key_type_clause(db_clause_list_t* clause_list, hsm_key_key_type_t key_type);
 
 /**
  * Create a clause for repository of a hsm key object and add it to a database clause list.
@@ -365,7 +367,7 @@ db_clause_t* hsm_key_key_type_clause(db_clause_list_t* clause_list, hsm_key_key_
  * \param[in] repository_text a character pointer.
  * \return a db_clause_t pointer to the added clause or NULL on error.
  */
-db_clause_t* hsm_key_repository_clause(db_clause_list_t* clause_list, const char* repository_text);
+extern db_clause_t* hsm_key_repository_clause(db_clause_list_t* clause_list, const char* repository_text);
 
 /**
  * Create a clause for backup of a hsm key object and add it to a database clause list.
@@ -376,14 +378,14 @@ db_clause_t* hsm_key_repository_clause(db_clause_list_t* clause_list, const char
  * \param[in] backup a hsm_key_backup_t.
  * \return a db_clause_t pointer to the added clause or NULL on error.
  */
-db_clause_t* hsm_key_backup_clause(db_clause_list_t* clause_list, hsm_key_backup_t backup);
+extern db_clause_t* hsm_key_backup_clause(db_clause_list_t* clause_list, hsm_key_backup_t backup);
 
 /**
  * Create a hsm key object in the database.
  * \param[in] hsm_key a hsm_key_t pointer.
  * \return DB_ERROR_* on failure, otherwise DB_OK.
  */
-int hsm_key_create(hsm_key_t* hsm_key);
+extern int hsm_key_create(hsm_key_t* hsm_key);
 
 /**
  * Get a hsm key object from the database by a id specified in `id`.
@@ -391,7 +393,7 @@ int hsm_key_create(hsm_key_t* hsm_key);
  * \param[in] id a db_value_t pointer.
  * \return DB_ERROR_* on failure, otherwise DB_OK.
  */
-int hsm_key_get_by_id(hsm_key_t* hsm_key, const db_value_t* id);
+extern int hsm_key_get_by_id(hsm_key_t* hsm_key, const db_value_t* id);
 
 /**
  * Get a hsm key object from the database by a locator specified in `locator`.
@@ -399,7 +401,7 @@ int hsm_key_get_by_id(hsm_key_t* hsm_key, const db_value_t* id);
  * \param[in] locator a character pointer.
  * \return DB_ERROR_* on failure, otherwise DB_OK.
  */
-int hsm_key_get_by_locator(hsm_key_t* hsm_key, const char* locator);
+extern int hsm_key_get_by_locator(hsm_key_t* hsm_key, const char* locator);
 
 /**
  * Get a new hsm key object from the database by a locator specified in `locator`.
@@ -407,14 +409,14 @@ int hsm_key_get_by_locator(hsm_key_t* hsm_key, const char* locator);
  * \param[in] locator a character pointer.
  * \return a hsm_key_t pointer or NULL on error or if it does not exist.
  */
-hsm_key_t* hsm_key_new_get_by_locator(const db_connection_t* connection, const char* locator);
+extern hsm_key_t* hsm_key_new_get_by_locator(const db_connection_t* connection, const char* locator);
 
 /**
  * Update a hsm key object in the database.
  * \param[in] hsm_key a hsm_key_t pointer.
  * \return DB_ERROR_* on failure, otherwise DB_OK.
  */
-int hsm_key_update(hsm_key_t* hsm_key);
+extern int hsm_key_update(hsm_key_t* hsm_key);
 
 /**
  * Count the number of hsm key objects in the database, if a selection of
@@ -426,7 +428,7 @@ int hsm_key_update(hsm_key_t* hsm_key);
  * should be counted.
  * \return DB_ERROR_* on failure, otherwise DB_OK.
  */
-int hsm_key_count(hsm_key_t* hsm_key, db_clause_list_t* clause_list, size_t* count);
+extern int hsm_key_count(hsm_key_t* hsm_key, db_clause_list_t* clause_list, size_t* count);
 
 /**
  * A list of hsm key objects.
@@ -450,14 +452,14 @@ struct hsm_key_list {
  * \param[in] connection a db_connection_t pointer.
  * \return a hsm_key_list_t pointer or NULL on error.
  */
-hsm_key_list_t* hsm_key_list_new(const db_connection_t* connection);
+extern hsm_key_list_t* hsm_key_list_new(const db_connection_t* connection);
 
 /**
  * Create a new hsm key object list that is a copy of another.
  * \param[in] hsm_key_list a hsm_key_list_t pointer.
  * \return a hsm_key_list_t pointer or NULL on error.
  */
-hsm_key_list_t* hsm_key_list_new_copy(const hsm_key_list_t* hsm_key_copy);
+extern hsm_key_list_t* hsm_key_list_new_copy(const hsm_key_list_t* hsm_key_copy);
 
 /**
  * Specify that objects should be stored within the list as they are fetch,
@@ -465,13 +467,13 @@ hsm_key_list_t* hsm_key_list_new_copy(const hsm_key_list_t* hsm_key_copy);
  * \param[in] hsm_key_list a hsm_key_list_t pointer.
  * \return DB_ERROR_* on failure, otherwise DB_OK.
  */
-int hsm_key_list_object_store(hsm_key_list_t* hsm_key_list);
+extern int hsm_key_list_object_store(hsm_key_list_t* hsm_key_list);
 
 /**
  * Delete a hsm key object list.
  * \param[in] hsm_key_list a hsm_key_list_t pointer.
  */
-void hsm_key_list_free(hsm_key_list_t* hsm_key_list);
+extern void hsm_key_list_free(hsm_key_list_t* hsm_key_list);
 
 /**
  * free global allocator.
@@ -483,7 +485,7 @@ void hsm_key_list_free(hsm_key_list_t* hsm_key_list);
  * \param[in] from_hsm_key_list a hsm_key_list_t pointer.
  * \return DB_ERROR_* on failure, otherwise DB_OK.
  */
-int hsm_key_list_copy(hsm_key_list_t* hsm_key_list, const hsm_key_list_t* from_hsm_key_list);
+extern int hsm_key_list_copy(hsm_key_list_t* hsm_key_list, const hsm_key_list_t* from_hsm_key_list);
 
 /**
  * Get hsm key objects from the database by a clause list.
@@ -491,7 +493,7 @@ int hsm_key_list_copy(hsm_key_list_t* hsm_key_list, const hsm_key_list_t* from_h
  * \param[in] clause_list a db_clause_list_t pointer.
  * \return DB_ERROR_* on failure, otherwise DB_OK.
  */
-int hsm_key_list_get_by_clauses(hsm_key_list_t* hsm_key_list, const db_clause_list_t* clause_list);
+extern int hsm_key_list_get_by_clauses(hsm_key_list_t* hsm_key_list, const db_clause_list_t* clause_list);
 
 /**
  * Get a new list of hsm key objects from the database by a clause list.
@@ -499,7 +501,7 @@ int hsm_key_list_get_by_clauses(hsm_key_list_t* hsm_key_list, const db_clause_li
  * \param[in] clause_list a db_clause_list_t pointer.
  * \return a hsm_key_list_t pointer or NULL on error.
  */
-hsm_key_list_t* hsm_key_list_new_get_by_clauses(const db_connection_t* connection, const db_clause_list_t* clause_list);
+extern hsm_key_list_t* hsm_key_list_new_get_by_clauses(const db_connection_t* connection, const db_clause_list_t* clause_list);
 
 /**
  * Get hsm key objects from the database by a policy_id specified in `policy_id`.
@@ -507,7 +509,7 @@ hsm_key_list_t* hsm_key_list_new_get_by_clauses(const db_connection_t* connectio
  * \param[in] policy_id a db_value_t pointer.
  * \return DB_ERROR_* on failure, otherwise DB_OK.
  */
-int hsm_key_list_get_by_policy_id(hsm_key_list_t* hsm_key_list, const db_value_t* policy_id);
+extern int hsm_key_list_get_by_policy_id(hsm_key_list_t* hsm_key_list, const db_value_t* policy_id);
 
 /**
  * Get a new list of hsm key objects from the database by a policy_id specified in `policy_id`.
@@ -515,7 +517,7 @@ int hsm_key_list_get_by_policy_id(hsm_key_list_t* hsm_key_list, const db_value_t
  * \param[in] policy_id a db_value_t pointer.
  * \return a hsm_key_list_t pointer or NULL on error.
  */
-hsm_key_list_t* hsm_key_list_new_get_by_policy_id(const db_connection_t* connection, const db_value_t* policy_id);
+extern hsm_key_list_t* hsm_key_list_new_get_by_policy_id(const db_connection_t* connection, const db_value_t* policy_id);
 
 /**
  * Get the first hsm key object in a hsm key object list and reset the
@@ -524,7 +526,7 @@ hsm_key_list_t* hsm_key_list_new_get_by_policy_id(const db_connection_t* connect
  * \return a hsm_key_t pointer or NULL on error or if there are no
  * hsm key objects in the hsm key object list.
  */
-const hsm_key_t* hsm_key_list_begin(hsm_key_list_t* hsm_key_list);
+extern const hsm_key_t* hsm_key_list_begin(hsm_key_list_t* hsm_key_list);
 
 /**
  * Get the first hsm key object in a hsm key object list and reset the
@@ -534,7 +536,7 @@ const hsm_key_t* hsm_key_list_begin(hsm_key_list_t* hsm_key_list);
  * \return a hsm_key_t pointer or NULL on error or if there are no
  * hsm key objects in the hsm key object list.
  */
-hsm_key_t* hsm_key_list_get_begin(hsm_key_list_t* hsm_key_list);
+extern hsm_key_t* hsm_key_list_get_begin(hsm_key_list_t* hsm_key_list);
 
 /**
  * Get the next hsm key object in a hsm key object list.
@@ -544,7 +546,7 @@ hsm_key_t* hsm_key_list_get_begin(hsm_key_list_t* hsm_key_list);
  * \return a hsm_key_t pointer or NULL on error or if there are no more
  * hsm key objects in the hsm key object list.
  */
-const hsm_key_t* hsm_key_list_next(hsm_key_list_t* hsm_key_list);
+extern const hsm_key_t* hsm_key_list_next(hsm_key_list_t* hsm_key_list);
 
 /**
  * Get the next hsm key object in a hsm key object list.
@@ -554,6 +556,6 @@ const hsm_key_t* hsm_key_list_next(hsm_key_list_t* hsm_key_list);
  * \return a hsm_key_t pointer or NULL on error or if there are no more
  * hsm key objects in the hsm key object list.
  */
-hsm_key_t* hsm_key_list_get_next(hsm_key_list_t* hsm_key_list);
+extern hsm_key_t* hsm_key_list_get_next(hsm_key_list_t* hsm_key_list);
 
 #endif
