@@ -1818,7 +1818,7 @@ updateZone(db_connection_t *dbconn, policy_t const *policy, zone_db_t* zone,
                      */
                     if (key_data_updated) {
                         if (key_data_update(keylist[i])) {
-                            ods_log_error("[%s] %s: key data update failed", module_str, scmd);
+                            ods_log_info("[%s] %s: key data update failed", module_str, scmd);
                             process = 0;
                             break;
                         }
@@ -1948,7 +1948,8 @@ getLastReusableKey(key_data_list_t *key_list, const policy_key_t *pkey)
 		if ((~hsm_key_role(hkey) & policy_key_role(pkey)) != 0 ||
 			/** hsmkey must be in use already. Allocating UNUSED keys is a
 			 * job for the keyfactory */
-			hkey->state == HSM_KEY_STATE_UNUSED )
+			hkey->state == HSM_KEY_STATE_UNUSED ||
+			hkey->state == HSM_KEY_STATE_DELETE )
 		{
 			hsm_key_free(hkey);
 			continue;
@@ -2666,7 +2667,11 @@ removeDeadKeys(db_connection_t *dbconn, key_data_t** keylist,
     int deleteCount = hsm_key_factory_delete_key(dbconn);
     ods_log_info("[%s] %s: keys deleted from HSM: %d", module_str, scmd, deleteCount);
 
-	return first_purge;
+    if(deleteCount > 0) {
+        return -1 - deleteCount;
+    } else {
+        return first_purge;
+    }
 }
 
 time_t
@@ -2801,6 +2806,11 @@ update(engine_type *engine, db_connection_t *dbconn, zone_db_t *zone, policy_t c
 	if (policy_keys_purge_after(policy) && keylist) {
 	    purge_return_time = removeDeadKeys(dbconn, keylist, keylist_size, deplist, now,
 	        policy_keys_purge_after(policy));
+            if(purge_return_time < -1) {
+                ods_log_info("[%s] reschedule enforcing policy due to deleting keys", module_str, scmd);
+                /* Keys have been deleted, we cannot continue in this same session, reschedule. */
+                return now + 60;
+            }
 	}
     
     
