@@ -54,7 +54,7 @@ cmdhandler_handle_cmd_help(int sockfd, cmdhandler_ctx_type* context, const char
         "sign --all                  Read all zones and schedule all for "
                                     "immediate (re-)sign.\n"
     );
-    client_printf(sockfd, buf);
+    client_printf(sockfd, "%s", buf);
 
     (void) snprintf(buf, ODS_SE_MAXLINE,
         "clear <zone>                Delete the internal storage of this "
@@ -65,7 +65,7 @@ cmdhandler_handle_cmd_help(int sockfd, cmdhandler_ctx_type* context, const char
         "flush                       Execute all scheduled tasks "
                                     "immediately.\n"
     );
-    client_printf(sockfd, buf);
+    client_printf(sockfd, "%s", buf);
 
     (void) snprintf(buf, ODS_SE_MAXLINE,
         "update <zone>               Update this zone signer "
@@ -79,7 +79,7 @@ cmdhandler_handle_cmd_help(int sockfd, cmdhandler_ctx_type* context, const char
         "stop                        Stop the engine.\n"
         "verbosity <nr>              Set verbosity.\n"
     );
-    client_printf(sockfd, buf);
+    client_printf(sockfd, "%s", buf);
     return 0;
 }
 
@@ -99,14 +99,14 @@ cmdhandler_handle_cmd_zones(int sockfd, cmdhandler_ctx_type* context, const char
     engine = getglobalcontext(context);
     if (!engine->zonelist || !engine->zonelist->zones) {
         (void)snprintf(buf, ODS_SE_MAXLINE, "There are no zones configured\n");
-        client_printf(sockfd, buf);
+        client_printf(sockfd, "%s", buf);
         return 0;
     }
     /* how many zones */
     pthread_mutex_lock(&engine->zonelist->zl_lock);
     (void)snprintf(buf, ODS_SE_MAXLINE, "There are %i zones configured\n",
         (int) engine->zonelist->zones->count);
-    client_printf(sockfd, buf);
+    client_printf(sockfd, "%s", buf);
     /* list zones */
     node = ldns_rbtree_first(engine->zonelist->zones);
     while (node && node != LDNS_RBTREE_NULL) {
@@ -115,7 +115,7 @@ cmdhandler_handle_cmd_zones(int sockfd, cmdhandler_ctx_type* context, const char
             buf[i] = 0;
         }
         (void)snprintf(buf, ODS_SE_MAXLINE, "- %s\n", zone->name);
-        client_printf(sockfd, buf);
+        client_printf(sockfd, "%s", buf);
         node = ldns_rbtree_next(node);
     }
     pthread_mutex_unlock(&engine->zonelist->zl_lock);
@@ -144,18 +144,18 @@ cmdhandler_handle_cmd_update(int sockfd, cmdhandler_ctx_type* context, const cha
         if (zl_changed == ODS_STATUS_UNCHANGED) {
             (void)snprintf(buf, ODS_SE_MAXLINE, "Zone list has not changed."
                 " Signer configurations updated.\n");
-            client_printf(sockfd, buf);
+            client_printf(sockfd, "%s", buf);
         } else if (zl_changed == ODS_STATUS_OK) {
             (void)snprintf(buf, ODS_SE_MAXLINE, "Zone list updated: %i "
             "removed, %i added, %i updated.\n",
                 engine->zonelist->just_removed,
                 engine->zonelist->just_added,
                 engine->zonelist->just_updated);
-            client_printf(sockfd, buf);
+            client_printf(sockfd, "%s", buf);
         } else {
             pthread_mutex_unlock(&engine->zonelist->zl_lock);
             (void)snprintf(buf, ODS_SE_MAXLINE, "Zone list has errors.\n");
-            client_printf(sockfd, buf);
+            client_printf(sockfd, "%s", buf);
         }
         if (zl_changed == ODS_STATUS_OK ||
             zl_changed == ODS_STATUS_UNCHANGED) {
@@ -184,7 +184,7 @@ cmdhandler_handle_cmd_update(int sockfd, cmdhandler_ctx_type* context, const cha
         if (!zone) {
             (void)snprintf(buf, ODS_SE_MAXLINE, "Error: Zone %s not found.\n",
                 cmdargument(cmd, NULL, ""));
-            client_printf(sockfd, buf);
+            client_printf(sockfd, "%s", buf);
             /* update all */
             cmdhandler_handle_cmd_update(sockfd, context, "update --all");
             return 1;
@@ -196,7 +196,7 @@ cmdhandler_handle_cmd_update(int sockfd, cmdhandler_ctx_type* context, const cha
 
         (void)snprintf(buf, ODS_SE_MAXLINE, "Zone %s config being updated.\n",
         cmdargument(cmd, NULL, ""));
-        client_printf(sockfd, buf);
+        client_printf(sockfd, "%s", buf);
         ods_log_verbose("[%s] zone %s scheduled for immediate update signconf",
             cmdh_str, cmdargument(cmd, NULL, ""));
         engine_wakeup_workers(engine);
@@ -231,12 +231,12 @@ cmdhandler_handle_cmd_retransfer(int sockfd, cmdhandler_ctx_type* context, const
     if (!zone) {
         (void)snprintf(buf, ODS_SE_MAXLINE, "Error: Zone %s not found.\n",
             cmdargument(cmd, NULL, ""));
-        client_printf(sockfd, buf);
+        client_printf(sockfd, "%s", buf);
     } else if (zone->adinbound->type != ADAPTER_DNS) {
         (void)snprintf(buf, ODS_SE_MAXLINE,
             "Error: Zone %s not configured to use DNS input adapter.\n",
             cmdargument(cmd, NULL, ""));
-        client_printf(sockfd, buf);
+        client_printf(sockfd, "%s", buf);
     } else {
         zone->xfrd->serial_retransfer = 1;
         xfrd_set_timer_now(zone->xfrd);
@@ -244,7 +244,7 @@ cmdhandler_handle_cmd_retransfer(int sockfd, cmdhandler_ctx_type* context, const
         dnshandler_fwd_notify(engine->dnshandler,
             (uint8_t*) ODS_SE_NOTIFY_CMD, strlen(ODS_SE_NOTIFY_CMD));
         (void)snprintf(buf, ODS_SE_MAXLINE, "Zone %s being re-transfered.\n", cmdargument(cmd, NULL, ""));
-        client_printf(sockfd, buf);
+        client_printf(sockfd, "%s", buf);
         ods_log_verbose("[%s] zone %s being re-transfered", cmdh_str, cmdargument(cmd, NULL, ""));
     }
     return 0;
@@ -312,20 +312,20 @@ cmdhandler_handle_cmd_sign(int sockfd, cmdhandler_ctx_type* context, const char
             if (strncmp(delim1+1, "--serial ", 9) != 0) {
                 (void)snprintf(buf, ODS_SE_MAXLINE, "Error: Expecting <zone> "
                     "--serial <nr>, got %s.\n", cmdargument(cmd, NULL, ""));
-                client_printf(sockfd, buf);
+                client_printf(sockfd, "%s", buf);
                 return -1;
             }
             delim2 = strchr(delim1+1, ' ');
             if (!delim2) {
                 (void)snprintf(buf, ODS_SE_MAXLINE, "Error: Expecting serial.\n");
-                client_printf(sockfd, buf);
+                client_printf(sockfd, "%s", buf);
                 return -1;
             }
             serial = (uint32_t) strtol(delim2+1, &end, 10);
             if (*end != '\0') {
                 (void)snprintf(buf, ODS_SE_MAXLINE, "Error: Expecting serial, "
                     "got %s.\n", delim2+1);
-                client_printf(sockfd, buf);
+                client_printf(sockfd, "%s", buf);
                 return -1;
             }
             force_serial = 1;
@@ -345,7 +345,7 @@ cmdhandler_handle_cmd_sign(int sockfd, cmdhandler_ctx_type* context, const char
         if (!zone) {
             (void)snprintf(buf, ODS_SE_MAXLINE, "Error: Zone %s not found.\n",
                 cmdargument(cmd, NULL, ""));
-            client_printf(sockfd, buf);
+            client_printf(sockfd, "%s", buf);
             return 1;
         }
 
@@ -433,7 +433,7 @@ cmdhandler_handle_cmd_clear(int sockfd, cmdhandler_ctx_type* context, const char
         ods_log_warning("[%s] cannot clear zone %s, zone not found",
             cmdh_str, cmdargument(cmd, NULL, ""));
     }
-    client_printf(sockfd, buf);
+    client_printf(sockfd, "%s", buf);
     return 0;
 }
 
@@ -456,7 +456,7 @@ cmdhandler_handle_cmd_queue(int sockfd, cmdhandler_ctx_type* context, const char
     engine = getglobalcontext(context);
     if (!engine->taskq || !engine->taskq->tasks) {
         (void)snprintf(buf, ODS_SE_MAXLINE, "There are no tasks scheduled.\n");
-        client_printf(sockfd, buf);
+        client_printf(sockfd, "%s", buf);
         return 0;
     }
     /* current time */
@@ -464,13 +464,13 @@ cmdhandler_handle_cmd_queue(int sockfd, cmdhandler_ctx_type* context, const char
     strtime = ctime(&now);
     (void)snprintf(buf, ODS_SE_MAXLINE, "It is now %s",
         strtime?strtime:"(null)");
-    client_printf(sockfd, buf);
+    client_printf(sockfd, "%s", buf);
     /* current work */
     pthread_mutex_lock(&engine->taskq->schedule_lock);
     /* how many tasks */
     (void)snprintf(buf, ODS_SE_MAXLINE, "\nThere are %i tasks scheduled.\n",
         (int) engine->taskq->tasks->count);
-    client_printf(sockfd, buf);
+    client_printf(sockfd, "%s", buf);
     /* list tasks */
     node = ldns_rbtree_first(engine->taskq->tasks);
     while (node && node != LDNS_RBTREE_NULL) {
@@ -479,7 +479,7 @@ cmdhandler_handle_cmd_queue(int sockfd, cmdhandler_ctx_type* context, const char
             buf[i] = 0;
         }
         taskdesc = schedule_describetask(task);
-        client_printf(sockfd, taskdesc);
+        client_printf(sockfd, "%s", taskdesc);
         free(taskdesc);
         node = ldns_rbtree_next(node);
     }
@@ -502,7 +502,7 @@ cmdhandler_handle_cmd_flush(int sockfd, cmdhandler_ctx_type* context, const char
     schedule_flush(engine->taskq);
     engine_wakeup_workers(engine);
     (void)snprintf(buf, ODS_SE_MAXLINE, "All tasks scheduled immediately.\n");
-    client_printf(sockfd, buf);
+    client_printf(sockfd, "%s", buf);
     ods_log_verbose("[%s] all tasks scheduled immediately", cmdh_str);
     return 0;
 }
@@ -524,7 +524,7 @@ cmdhandler_handle_cmd_reload(int sockfd, cmdhandler_ctx_type* context, const cha
     pthread_cond_signal(&engine->signal_cond);
     pthread_mutex_unlock(&engine->signal_lock);
     (void)snprintf(buf, ODS_SE_MAXLINE, "Reloading engine.\n");
-    client_printf(sockfd, buf);
+    client_printf(sockfd, "%s", buf);
     return 0;
 }
 
@@ -544,7 +544,7 @@ cmdhandler_handle_cmd_stop(int sockfd, cmdhandler_ctx_type* context, const char
     pthread_cond_signal(&engine->signal_cond);
     pthread_mutex_unlock(&engine->signal_lock);
     (void)snprintf(buf, ODS_SE_MAXLINE, ODS_SE_STOP_RESPONSE);
-    client_printf(sockfd, buf);
+    client_printf(sockfd, "%s", buf);
     return 0;
 }
 
@@ -558,7 +558,7 @@ cmdhandler_handle_cmd_start(int sockfd, cmdhandler_ctx_type* context, const char
 {
     char buf[ODS_SE_MAXLINE];
     (void)snprintf(buf, ODS_SE_MAXLINE, "Engine already running.\n");
-    client_printf(sockfd, buf);
+    client_printf(sockfd, "%s", buf);
     return 0;
 }
 
@@ -572,7 +572,7 @@ cmdhandler_handle_cmd_running(int sockfd, cmdhandler_ctx_type* context, const ch
 {
     char buf[ODS_SE_MAXLINE];
     (void)snprintf(buf, ODS_SE_MAXLINE, "Engine running.\n");
-    client_printf(sockfd, buf);
+    client_printf(sockfd, "%s", buf);
     return 0;
 }
 
@@ -589,7 +589,7 @@ cmdhandler_handle_cmd_verbosity(int sockfd, cmdhandler_ctx_type* context, const
     val = atoi(cmdargument(cmd, NULL, "1"));
     ods_log_setverbosity(val);
     (void)snprintf(buf, ODS_SE_MAXLINE, "Verbosity level set to %i.\n", val);
-    client_printf(sockfd, buf);
+    client_printf(sockfd, "%s", buf);
     return 0;
 }
 
@@ -603,7 +603,7 @@ cmdhandler_handle_cmd_error(int sockfd, cmdhandler_ctx_type* context, const char
 {
     char buf[ODS_SE_MAXLINE];
     (void)snprintf(buf, ODS_SE_MAXLINE, "Error: %s.\n", str?str:"(null)");
-    client_printf(sockfd, buf);
+    client_printf(sockfd, "%s", buf);
 }
 
 
@@ -617,10 +617,58 @@ cmdhandler_handle_cmd_unknown(int sockfd, cmdhandler_ctx_type* context, const ch
     char buf[ODS_SE_MAXLINE];
     (void)snprintf(buf, ODS_SE_MAXLINE, "Unknown command %s.\n",
         str?str:"(null)");
-    client_printf(sockfd, buf);
+    client_printf(sockfd, "%s", buf);
 }
 
 
+/**
+ * Handle the 'time leap' command.
+ *
+ */
+static int
+cmdhandler_handle_cmd_timeleap(int sockfd, cmdhandler_ctx_type* context, char *cmd)
+{
+    struct tm strtime_struct;
+    char strtime[64]; /* at least 26 according to docs plus a long integer */
+    time_t now = time_now();
+    time_t time_leap = 0;
+    time_t next_leap = 0;
+    struct tm tm;
+    int taskcount;
+    engine_type* engine = getglobalcontext(context);
+
+    /* skip "time" and "leap" */
+    while(isspace(*cmd)) ++cmd;
+    cmd = &cmd[4];
+    while(isspace(*cmd)) ++cmd;
+    cmd = &cmd[4];
+    while(isspace(*cmd)) ++cmd;
+
+    if (strptime(cmd, "%Y-%m-%d-%H:%M:%S", &tm)) {
+        tm.tm_isdst = -1;
+        time_leap = mktime(&tm);
+        client_printf(sockfd, "Using %s parameter value as time to leap to\n", cmd);
+    } else {
+        client_printf_err(sockfd, "Time leap: Error - could not convert '%s' to a time. Format is YYYY-MM-DD-HH:MM:SS \n", cmd);
+        return -1;
+    }
+    if (!engine->taskq || !engine->taskq->tasks) {
+        client_printf(sockfd, "There are no tasks scheduled.\n");
+        return 1;
+    }
+    schedule_info(engine->taskq, &next_leap, NULL, &taskcount);
+    now = time_now();
+    strftime(strtime, sizeof (strtime), "%c", localtime_r(&now, &strtime_struct));
+    client_printf(sockfd, "There are %i tasks scheduled.\nIt is now       %s (%ld seconds since epoch)\n", taskcount, strtime, (long) now);
+    set_time_now(time_leap);
+    strftime(strtime, sizeof (strtime), "%c", localtime_r(&time_leap, &strtime_struct));
+    client_printf(sockfd, "Leaping to time %s (%ld seconds since epoch)\n", (strtime[0] ? strtime : "(null)"), (long) time_leap);
+    ods_log_info("Time leap: Leaping to time %s\n", strtime);
+    client_printf(sockfd, "Waking up workers\n");
+    engine_wakeup_workers(engine);
+    return 0;
+}
+
 struct cmd_func_block helpCmdDef = { "help", NULL, NULL, NULL, &cmdhandler_handle_cmd_help };
 struct cmd_func_block zonesCmdDef = { "zones", NULL, NULL, NULL, &cmdhandler_handle_cmd_zones };
 struct cmd_func_block signCmdDef = { "sign", NULL, NULL, NULL, &cmdhandler_handle_cmd_sign };
@@ -634,6 +682,7 @@ struct cmd_func_block reloadCmdDef = { "reload", NULL, NULL, NULL, &cmdhandler_h
 struct cmd_func_block retransferCmdDef = { "retransfer", NULL, NULL, NULL, &cmdhandler_handle_cmd_retransfer };
 struct cmd_func_block runningCmdDef = { "running", NULL, NULL, NULL, &cmdhandler_handle_cmd_running };
 struct cmd_func_block verbosityCmdDef = { "verbosity", NULL, NULL, NULL, &cmdhandler_handle_cmd_verbosity };
+struct cmd_func_block timeleapCmdDef = { "time leap", NULL, NULL, NULL, &cmdhandler_handle_cmd_timeleap };
 
 struct cmd_func_block* signcommands[] = {
     &helpCmdDef,
@@ -649,6 +698,7 @@ struct cmd_func_block* signcommands[] = {
     &retransferCmdDef,
     &runningCmdDef,
     &verbosityCmdDef,
+    &timeleapCmdDef,
     NULL
 };
 struct cmd_func_block** signercommands = signcommands;
